//package src;

import java.util.*;

public class ActionRule {

    private ArrayList<String> flexibleAttributesFrom;
    private ArrayList<String> flexibleAttributesTo;
    private ArrayList<String> stableAttributes;
    private String decisionAttribute;
    private String decisionFrom;
    private String decisionTo;
    
    private float confidence;
    private int support;

    public ActionRule(
        ArrayList<String> flexibleAttributesFrom,
        ArrayList<String> flexibleAttributesTo,
        ArrayList<String> stableAttributes,
        String decisionAttribute,
        String decisionTo,
        String decisionFrom
    ) {
        this.flexibleAttributesFrom = flexibleAttributesFrom;
        this.flexibleAttributesTo = flexibleAttributesTo;
        this.stableAttributes = stableAttributes;
        this.decisionAttribute = decisionAttribute;
        this.decisionTo = decisionTo;
        this.decisionFrom = decisionFrom;

        this.confidence = 0.0f;
        this.support = 0;
    }

    public ArrayList<String> getY1() {
        ArrayList<String> temp = new ArrayList<String>(this.flexibleAttributesFrom);
        temp.addAll(this.stableAttributes);
        return temp;
    }

    public ArrayList<String> getY2() {
        ArrayList<String> temp = new ArrayList<String>(this.flexibleAttributesTo);
        temp.addAll(this.stableAttributes);
        return temp;
    }

    public ArrayList<String> getY1Z1() {
        ArrayList<String> temp = this.getY1();
        temp.add(this.decisionFrom);
        return temp;
    }

    public ArrayList<String> getY2Z2() {
        ArrayList<String> temp = this.getY2();
        temp.add(this.decisionTo);
        return temp;
    }

    public void setSupport(int support) {
        this.support = support;
    }

    public int getSupport() {
        return this.support;
    }

    public void setConfidence(float confidence) {
        this.confidence = confidence;
    }

    public float getConfidence() {
        return this.confidence;
    }

    public String toString() {
        String flexibleRules = "";
        String stableRules = "";

        Collections.sort(this.flexibleAttributesTo);
        Collections.sort(this.flexibleAttributesFrom);
        
        for (int i = 0; i < this.flexibleAttributesTo.size(); i ++) {
            String name = "" + this.flexibleAttributesTo.get(i).split(IOHandler.ATTRIBUTE_DELIMITER)[0];
            flexibleRules += "(" + name + ", " + 
                this.flexibleAttributesFrom.get(i) + " → " +
                this.flexibleAttributesTo.get(i) + ")";

            if (i != this.flexibleAttributesTo.size() - 1) {
                flexibleRules += " ^ ";
            }
        }

        for (int i = 0; i < this.stableAttributes.size(); i ++) {
            stableRules += this.stableAttributes.get(i);

            if (i != this.stableAttributes.size() - 1) {
                stableRules += " ^ ";
            }
        }

        String rules = flexibleRules;
        if (stableRules.length() > 0) {
            rules += " ^ " + stableRules;
        }

        return "[" + rules + "] → (" + this.decisionAttribute + ", " + 
            this.decisionFrom + " → " + this.decisionTo + ")" +
            " = (" + this.support + ", " + (this.confidence * 100) + "%)";
    }

    public boolean equals(Object obj){
        if(obj instanceof ActionRule){
            return obj.toString().equals(this.toString());
        }

        return false;
    }
}