//package src;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.*;

public class IOHandler {
    public static final String ATTRIBUTE_DELIMITER = "~";

    public List<String> attributeNames;
    public HashMap<HashSet<String>, HashSet<String>> attributeValues;
    public HashMap<String, HashSet<String>> distinctAttributeValues;

    public IOHandler(List<String> attributeNames) {
        this.attributeNames = attributeNames;
        this.attributeValues = new HashMap<HashSet<String>, HashSet<String>>();
        this.distinctAttributeValues = new HashMap<String, HashSet<String>>();
    }

    public ArrayList<ArrayList<String>> readData(String filename) {
        ArrayList<ArrayList<String>> data = new ArrayList<ArrayList<String>>();
        try {
            Scanner input = new Scanner(new File(filename));
            int lineNo = 0;

            while (input.hasNextLine()) {
                String line = input.nextLine();
                String[] lineData;
                if (line.indexOf(",") != -1) {
                    lineData = line.split(",");
                } else {
                    lineData = line.split("\t");
                }
                String key;

                lineNo++;
                ArrayList<String> tempList = new ArrayList<String>();
                HashSet<String> set;

                for (int i = 0; i < lineData.length; i++) {
                    String currentAttributeValue = lineData[i];
                    String attributeName = this.attributeNames.get(i);
                    key = attributeName + IOHandler.ATTRIBUTE_DELIMITER +
                        currentAttributeValue;

                    tempList.add(key);

                    HashSet<String> mapKey = new HashSet<String>();
                    mapKey.add(key);
                    setMap(this.attributeValues, lineData[i], mapKey, lineNo);

                    if (this.distinctAttributeValues.containsKey(attributeName)) {
                        set = this.distinctAttributeValues.get(attributeName);
                        set.add(key);

                    } else {
                        set = new HashSet<String>();
                    }

                    set.add(key);
                    this.distinctAttributeValues.put(attributeName, set);
                }

                data.add(tempList);
            }


        } catch (FileNotFoundException e) {
            System.out.println("File Not Found");
            e.printStackTrace();
        }

        return data;
    }

    private void setMap(Map<HashSet<String>, HashSet<String>> values,
                               String string, HashSet<String> key, int lineNo) {
        HashSet<String> tempSet;

        if (values.containsKey(key)) {
            tempSet = values.get(key);
        } else {
            tempSet = new HashSet<String>();
        }

        tempSet.add("x" + lineNo);
        values.put(key, tempSet);
    }

    public void saveOutput(String outputData, String filename) throws IOException {
        File file = new File(filename);

        FileWriter fileWriter = new FileWriter(file);
        fileWriter.write(outputData);
        fileWriter.close();
    }
}