//package src;

import java.util.*;
import java.io.IOException;

public class Main {
    public static void main(String[] args) throws IOException, Exception {
        test();

        GUI object = new GUI();
        // run(
        //     // "./data.txt",
        //     // "./attribute.txt",
        //     // "c",
        //     // "f",
        //     "./data/mammographic/mammographic_masses.data",
        //     "./data/mammographic/mammographic_masses.attributes",
        //     "Age",
        //     "Severity",
        //     "0",
        //     "1",
        //     "3",
        //     "0.1"
        // );
    }

    public static void run(
        String dataFile, // data file
        String attributesFile, // attributefile
        String stableAttributes,
        String decision, // decision
        String decisionTo, // to
        String decisionFrom, // from
        String minSupport,
        String minConfidence
    ) throws IOException, Exception {
        Main.run(
            dataFile,
            attributesFile,
            (List<String>) Arrays.asList(stableAttributes.split(",")),
            decision,
            decision + IOHandler.ATTRIBUTE_DELIMITER + decisionTo,
            decision + IOHandler.ATTRIBUTE_DELIMITER + decisionFrom,
            Integer.parseInt(minSupport),
            Float.parseFloat(minConfidence)
        );
    }

    public static void run(
        String dataFile,
        String attributesFile,
        List<String> stableAttributes,
        String decision,
        String decisionTo,
        String decisionFrom,
        int minSupport,
        float minConfidence
    ) throws IOException, Exception {

        SecondFrame secondFrame = new SecondFrame();

        AttributesHandler attributesHandler = new AttributesHandler();
        List<String> attributeNames = attributesHandler.readAttributes(attributesFile);

        System.out.println(attributeNames);
        //Read data
        IOHandler ioHandler = new IOHandler(attributeNames);
        ArrayList<ArrayList<String>> data = ioHandler.readData(dataFile);
        attributesHandler.setAttributeValues(ioHandler.attributeValues);
        attributesHandler.setDistinctAttributeValues(ioHandler.distinctAttributeValues);

        //Set stable, flexible and decision attributes
        attributesHandler.stableAttributes = stableAttributes;
        attributesHandler.setDecisionAttribute(
            decision,
            decisionTo,
            decisionFrom
        );
        for (String attribute : attributeNames) {
            if (!attribute.equals(decision) 
                && stableAttributes.indexOf(attribute) == -1) {
                attributesHandler.flexibleAttributes.add(attribute);
            }
        }

        long start = System.currentTimeMillis();
        //Find Certain and Possible rules
        RuleHandler ruleHandler = new RuleHandler(
            new LERSHandler(data),
            ioHandler.distinctAttributeValues
            // secondFrame
        );
        ruleHandler.findRules(ioHandler.attributeValues, attributesHandler.decisionValues);
        HashMap<ArrayList<String>, String> certainRules = ruleHandler.GetCertainRules();

        secondFrame.addOutput("Searching for Action Rules");
        ActionRuleStrategy strategy = new ActionRuleStrategy(
            data,
            certainRules,
            attributesHandler.flexibleAttributes,
            attributesHandler.stableAttributes,
            attributesHandler.decisionAttribute,
            attributesHandler.decisionTo,
            attributesHandler.decisionFrom
        );

        long lersTime = System.currentTimeMillis() - start;
        secondFrame.addOutput("Total certain rules to check for " + certainRules.size() + " action rules found.");
        strategy.run(minSupport, minConfidence);
        ArrayList<ActionRule> rules = strategy.getActionRules();
        secondFrame.addOutput(rules.size() + " action rules found.");
        secondFrame.addOutput("Action rules: ");
        for(int i = 0; i < rules.size(); i++) {
            secondFrame.addOutput(rules.get(i).toString());
        }

        // Saving the output file
        ioHandler.saveOutput(secondFrame.getOutput(), "Output.txt");
        long time = System.currentTimeMillis() - start;
        secondFrame.addOutput(String.format("\nThe code ran in %.2f seconds", time / 1000.0));
        secondFrame.addOutput(String.format("Lers time is %.2f seconds\n", lersTime / 1000.0));
    }

    public static void test() {
        ActionRuleStrategy.test();
        Attribute.test();
    }
}
