//package src;

import java.util.*;

public class RuleHandler {

    private HashMap<ArrayList<String>, String> certainRules;
    private Map<ArrayList<String>, HashSet<String>> possibleRules;

    private Map<ArrayList<String>, HashSet<String>> markedValues;
    private Map<String, HashSet<String>> distinctAttributeValues;
    private Map<HashSet<String>, HashSet<String>> reducedAttributeValues;
    private LERSHandler lersHandler;
    private SecondFrame secondFrame;
    private String output;

    public RuleHandler(
        LERSHandler lersHandler, 
        HashMap<String, HashSet<String>> distinctAttributeValues,
        SecondFrame secondFrame
    ) {
        this(lersHandler, distinctAttributeValues);
        this.secondFrame = secondFrame;
    }

    public RuleHandler(
        LERSHandler lersHandler, 
        HashMap<String, HashSet<String>> distinctAttributeValues
    ) {
        this.lersHandler = lersHandler;
        this.distinctAttributeValues = distinctAttributeValues;
        this.certainRules = new HashMap<ArrayList<String>, String>();
        this.markedValues = new HashMap<ArrayList<String>, HashSet<String>>();
        this.possibleRules = new HashMap<ArrayList<String>, HashSet<String>>();
        this.reducedAttributeValues = new HashMap<HashSet<String>, HashSet<String>>();
        this.output = "";
    }

    public void findRules(
        Map<HashSet<String>, HashSet<String>> attributeValues, 
        Map<String, HashSet<String>> decisionValues
    ) {
        int loopCount = 0;

        while (!attributeValues.isEmpty()) {
            print("\nLoop " + (++loopCount) + ":");
            print("--------------------------");

            for (Map.Entry<HashSet<String>, HashSet<String>> set : attributeValues.entrySet()) {
                ArrayList<String> setKey = new ArrayList<String>();
                setKey.addAll(set.getKey());

                if (set.getValue().isEmpty()) {
                    continue;
                } else {
                    for (Map.Entry<String, HashSet<String>> decisionSet : decisionValues.entrySet()) {
                        if (decisionSet.getValue().containsAll(set.getValue())) {
                            this.certainRules.put(setKey, decisionSet.getKey());
                            this.markedValues.put(setKey, set.getValue());
                            break;
                        }
                    }
                }

                if (!this.markedValues.containsKey(setKey)) {
                    HashSet<String> possibleRulesSet = new HashSet<String>();
                    for (Map.Entry<String, HashSet<String>> decisionSet : decisionValues.entrySet()) {
                        possibleRulesSet.add(decisionSet.getKey());
                    }
                    this.possibleRules.put(setKey, possibleRulesSet);
                }
            }

            this.removeMarkedValues(attributeValues);

            this.printCertainRulesMap(this.certainRules);
            print("--------------------------");

            this.combinePossibleRules(attributeValues);
        }
    }

    private void removeMarkedValues(
        Map<HashSet<String>, HashSet<String>> attributeValues
    ) {
        for (Map.Entry<ArrayList<String>, HashSet<String>> markedSet : this.markedValues.entrySet()) {
            attributeValues.remove(new HashSet<String>(markedSet.getKey()));
        }
    }

    private void printCertainRulesMap(Map<ArrayList<String>, String> value) {
        print("\nCertain Rules:");
        for (Map.Entry<ArrayList<String>, String> set : value.entrySet()) {
            int support = this.lersHandler.calculateSupportLERS(set.getKey(), set.getValue());
            String confidence = this.lersHandler.calculateConfidenceLERS(set.getKey(), set.getValue());

            print(
                set.getKey().toString() + " -> " + 
                set.getValue() + "[Support: " + 
                support + ", Confidence: " + 
                confidence + "%]"
            );

        }
    }

    private void printPossibleRulesMap(
        Map<ArrayList<String>, HashSet<String>> value
    ) {
        if (!value.isEmpty()) {
            print("\nPossible Rules:");
            for (Map.Entry<ArrayList<String>, HashSet<String>> set : value.entrySet()) {
                for (String possibleValue : set.getValue()) {
                    int support = this.lersHandler.calculateSupportLERS(
                        set.getKey(), 
                        possibleValue
                    );
                    String confidence = this.lersHandler.calculateConfidenceLERS(
                        set.getKey(), 
                        possibleValue
                    );

                    print(
                        set.getKey().toString() + " -> " + 
                        possibleValue + "[Support: " + support + 
                        ", Confidence: " + confidence + 
                        "%]"
                    );
                }

            }
        }
    }

    private void combinePossibleRules(
        Map<HashSet<String>, HashSet<String>> attributeValues
    ) {
        Set<ArrayList<String>> keySet = this.possibleRules.keySet();
        ArrayList<ArrayList<String>> keyList = new ArrayList<ArrayList<String>>();
        keyList.addAll(keySet);

        for (int i = 0; i < this.possibleRules.size(); i++) {
            for (int j = (i + 1); j < this.possibleRules.size(); j++) {
                HashSet<String> combinedKeys = new HashSet<String>(keyList.get(i));
                combinedKeys.addAll(new HashSet<String>(keyList.get(j)));

                if (!this.checkSameGroup(combinedKeys)) {
                    this.combineAttributeValues(combinedKeys, attributeValues);
                }
            }
        }

        this.possibleRules.clear();
        this.removeRedundantValues();
        this.clearAttributeValues(attributeValues);
    }

    private boolean checkSameGroup(HashSet<String> combinedKeys) {
        ArrayList<String> keyList = new ArrayList<String>();

        for (String combinedKey : combinedKeys) {
            for (Map.Entry<String, HashSet<String>> singleAttribute : this.distinctAttributeValues.entrySet()) {
                if (singleAttribute.getValue().contains(combinedKey)) {
                    if (keyList.contains(singleAttribute.getKey())) {
                        return true;
                    } else {
                        keyList.add(singleAttribute.getKey());
                    }
                } else {
                    continue;
                }
            }
        }

        return false;
    }

    private void combineAttributeValues(
        HashSet<String> combinedKeys,
        Map<HashSet<String>, HashSet<String>> attributeValues
    ) {
        HashSet<String> combinedValues = new HashSet<String>();

        for (Map.Entry<HashSet<String>, HashSet<String>> attributeValue : attributeValues.entrySet()) {
            if (combinedKeys.containsAll(attributeValue.getKey())) {
                if (combinedValues.isEmpty()) {
                    combinedValues.addAll(attributeValue.getValue());
                } else {
                    combinedValues.retainAll(attributeValue.getValue());
                }
            }
        }
        this.reducedAttributeValues.put(combinedKeys, combinedValues);

    }

    private void removeRedundantValues() {
        HashSet<HashSet<String>> mark = new HashSet<HashSet<String>>();

        for (Map.Entry<HashSet<String>, HashSet<String>> reducedAttributeValue : this.reducedAttributeValues.entrySet()) {
            for (Map.Entry<ArrayList<String>, String> attributeValue : this.certainRules.entrySet()) {
                if (reducedAttributeValue.getKey().containsAll(attributeValue.getKey()) || reducedAttributeValue.getValue().isEmpty()) {
                    //print(reducedAttributeValue.getKey().toString() + " is already marked!");

                    mark.add(reducedAttributeValue.getKey());
                }
            }
        }

        for (HashSet<String> toRemove : mark) {
            this.reducedAttributeValues.remove(toRemove);
        }


    }

    private void clearAttributeValues(Map<HashSet<String>, HashSet<String>> attributeValues) {
        attributeValues.clear();
        for (Map.Entry<HashSet<String>, HashSet<String>> reducedAttributeValue : this.reducedAttributeValues.entrySet()) {
            attributeValues.put(reducedAttributeValue.getKey(), reducedAttributeValue.getValue());
        }
        this.reducedAttributeValues.clear();
    }


    private void printAttributeMap(Map<HashSet<String>, HashSet<String>> values) {
        for (Map.Entry<HashSet<String>, HashSet<String>> set : values.entrySet()) {
            print(set.getKey().toString() + " = " + set.getValue());
        }
    }

    public HashMap<ArrayList<String>, String> GetCertainRules() {
        return this.certainRules;
    }

    private void print(String output) {
        if (this.secondFrame != null) {
            this.output += output;
        } else {
            System.out.println(output);
        }
    }
}
